// Copyright 2023 Google LLC
//
// Licensed under the Apache License, Version 2.0 (the "License");
// you may not use this file except in compliance with the License.
// You may obtain a copy of the License at
//
//     http://www.apache.org/licenses/LICENSE-2.0
//
// Unless required by applicable law or agreed to in writing, software
// distributed under the License is distributed on an "AS IS" BASIS,
// WITHOUT WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied.
// See the License for the specific language governing permissions and
// limitations under the License.

syntax = "proto3";

package google.ads.googleads.v14.services;

import "google/ads/googleads/v14/common/policy.proto";
import "google/ads/googleads/v14/enums/response_content_type.proto";
import "google/ads/googleads/v14/resources/ad_group_criterion.proto";
import "google/api/annotations.proto";
import "google/api/client.proto";
import "google/api/field_behavior.proto";
import "google/api/resource.proto";
import "google/protobuf/field_mask.proto";
import "google/rpc/status.proto";

option csharp_namespace = "Google.Ads.GoogleAds.V14.Services";
option go_package = "google.golang.org/genproto/googleapis/ads/googleads/v14/services;services";
option java_multiple_files = true;
option java_outer_classname = "AdGroupCriterionServiceProto";
option java_package = "com.google.ads.googleads.v14.services";
option objc_class_prefix = "GAA";
option php_namespace = "Google\\Ads\\GoogleAds\\V14\\Services";
option ruby_package = "Google::Ads::GoogleAds::V14::Services";

// Proto file describing the Ad Group Criterion service.

// Service to manage ad group criteria.
service AdGroupCriterionService {
  option (google.api.default_host) = "googleads.googleapis.com";
  option (google.api.oauth_scopes) = "https://www.googleapis.com/auth/adwords";

  // Creates, updates, or removes criteria. Operation statuses are returned.
  //
  // List of thrown errors:
  //   [AdGroupCriterionError]()
  //   [AdxError]()
  //   [AuthenticationError]()
  //   [AuthorizationError]()
  //   [BiddingError]()
  //   [BiddingStrategyError]()
  //   [CollectionSizeError]()
  //   [ContextError]()
  //   [CriterionError]()
  //   [DatabaseError]()
  //   [DateError]()
  //   [DistinctError]()
  //   [FieldError]()
  //   [FieldMaskError]()
  //   [HeaderError]()
  //   [IdError]()
  //   [InternalError]()
  //   [MultiplierError]()
  //   [MutateError]()
  //   [NewResourceCreationError]()
  //   [NotEmptyError]()
  //   [NullError]()
  //   [OperationAccessDeniedError]()
  //   [OperatorError]()
  //   [PolicyViolationError]()
  //   [QuotaError]()
  //   [RangeError]()
  //   [RequestError]()
  //   [ResourceCountLimitExceededError]()
  //   [SizeLimitError]()
  //   [StringFormatError]()
  //   [StringLengthError]()
  //   [UrlFieldError]()
  rpc MutateAdGroupCriteria(MutateAdGroupCriteriaRequest)
      returns (MutateAdGroupCriteriaResponse) {
    option (google.api.http) = {
      post: "/v14/customers/{customer_id=*}/adGroupCriteria:mutate"
      body: "*"
    };
    option (google.api.method_signature) = "customer_id,operations";
  }
}

// Request message for
// [AdGroupCriterionService.MutateAdGroupCriteria][google.ads.googleads.v14.services.AdGroupCriterionService.MutateAdGroupCriteria].
message MutateAdGroupCriteriaRequest {
  // Required. ID of the customer whose criteria are being modified.
  string customer_id = 1 [(google.api.field_behavior) = REQUIRED];

  // Required. The list of operations to perform on individual criteria.
  repeated AdGroupCriterionOperation operations = 2
      [(google.api.field_behavior) = REQUIRED];

  // If true, successful operations will be carried out and invalid
  // operations will return errors. If false, all operations will be carried
  // out in one transaction if and only if they are all valid.
  // Default is false.
  bool partial_failure = 3;

  // If true, the request is validated but not executed. Only errors are
  // returned, not results.
  bool validate_only = 4;

  // The response content type setting. Determines whether the mutable resource
  // or just the resource name should be returned post mutation.
  google.ads.googleads.v14.enums.ResponseContentTypeEnum.ResponseContentType
      response_content_type = 5;
}

// A single operation (create, remove, update) on an ad group criterion.
message AdGroupCriterionOperation {
  // FieldMask that determines which resource fields are modified in an update.
  google.protobuf.FieldMask update_mask = 4;

  // The list of policy violation keys that should not cause a
  // PolicyViolationError to be reported. Not all policy violations are
  // exemptable, refer to the is_exemptible field in the returned
  // PolicyViolationError.
  //
  // Resources violating these polices will be saved, but will not be eligible
  // to serve. They may begin serving at a later time due to a change in
  // policies, re-review of the resource, or a change in advertiser
  // certificates.
  repeated google.ads.googleads.v14.common.PolicyViolationKey
      exempt_policy_violation_keys = 5;

  // The mutate operation.
  oneof operation {
    // Create operation: No resource name is expected for the new criterion.
    google.ads.googleads.v14.resources.AdGroupCriterion create = 1;

    // Update operation: The criterion is expected to have a valid resource
    // name.
    google.ads.googleads.v14.resources.AdGroupCriterion update = 2;

    // Remove operation: A resource name for the removed criterion is expected,
    // in this format:
    //
    // `customers/{customer_id}/adGroupCriteria/{ad_group_id}~{criterion_id}`
    string remove = 3 [(google.api.resource_reference) = {
      type: "googleads.googleapis.com/AdGroupCriterion"
    }];
  }
}

// Response message for an ad group criterion mutate.
message MutateAdGroupCriteriaResponse {
  // Errors that pertain to operation failures in the partial failure mode.
  // Returned only when partial_failure = true and all errors occur inside the
  // operations. If any errors occur outside the operations (for example, auth
  // errors), we return an RPC level error.
  google.rpc.Status partial_failure_error = 3;

  // All results for the mutate.
  repeated MutateAdGroupCriterionResult results = 2;
}

// The result for the criterion mutate.
message MutateAdGroupCriterionResult {
  // Returned for successful operations.
  string resource_name = 1 [(google.api.resource_reference) = {
    type: "googleads.googleapis.com/AdGroupCriterion"
  }];

  // The mutated ad group criterion with only mutable fields after mutate. The
  // field will only be returned when response_content_type is set to
  // "MUTABLE_RESOURCE".
  google.ads.googleads.v14.resources.AdGroupCriterion ad_group_criterion = 2;
}
